//
// This file is part of the FLTK Imago project
// Copyright (C) 2002-2005 Chris Osgood
// http://imago.functionalfuture.com
//
// "$Id: Fl_SVG_Button.cxx 46 2005-01-04 04:09:52Z chris $"
//
// Fl_SVG_Button routines.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "imago at functionalfuture.com"
//

#ifdef _WIN32
// The following include is a nasty little hack for Windows; otherwise you will
// get unresolved symbols from the Fl:: class.  This basically applies
// __declspec(dllimport) to "class Fl".  However, you can't just define FL_DLL
// for the whole project  otherwise you will get multiple symbols.  This is
// partly the fault of the MS compiler but it could be fixed in FLTK.
#define FL_DLL
#include <FL/Fl.H>
#undef FL_DLL
#undef FL_EXPORT
#undef Fl_Export_H
#include <FL/Fl_Export.H>
#else
#include <FL/Fl.H>
#endif

#include <FL/fl_draw.H>
#include "Fl_SVG_Button.H"

/**
 * Button constructor
 * @param X X position
 * @param Y Y position
 * @param W Width
 * @param H Height
 * @param label Label text
 */
Fl_SVG_Button::Fl_SVG_Button(int X, int Y, int W, int H, const char* label)
   : Fl_Button(X, Y, W, H, label), down_image_(0), focus_image_(0),
     highlight_image_(0), focus_highlight_(false)
{
}

/**
 * Destructor
 */
Fl_SVG_Button::~Fl_SVG_Button()
{
   if (up_image_)
      delete up_image_;

   if (down_image_)
      delete down_image_;

   if (focus_image_)
      delete focus_image_;

   if (highlight_image_)
      delete highlight_image_;
}

/**
 * Sets the normal image
 * @param img Image
 */
void Fl_SVG_Button::up_image(Fl_SVG_Image* img)
{
   img->size(w(), h()); 
   img->bgcolor(color());
   up_image_ = img;
}

/**
 * Sets the pressed image
 * @param img Image
 */
void Fl_SVG_Button::down_image(Fl_SVG_Image* img)
{
   img->size(w(), h()); 
   img->bgcolor(color());
   down_image_ = img;
}

/**
 * Sets the "widget has focus" image
 * @param img Image
 */
void Fl_SVG_Button::focus_image(Fl_SVG_Image* img)
{
   img->size(w(), h()); 
   img->bgcolor(color());
   focus_image_ = img;
}

/**
 * Sets the "mouse over widget" image
 * @param img Image
 */
void Fl_SVG_Button::highlight_image(Fl_SVG_Image* img)
{
   img->size(w(), h()); 
   img->bgcolor(color());
   highlight_image_ = img;
}

/**
 * Standard FLTK draw method.
 */
void Fl_SVG_Button::draw()
{
   if (type() == FL_HIDDEN_BUTTON) return;
   Fl_Color col = value() ? selection_color() : color();

   draw_box(FL_FLAT_BOX, col);

   fl_push_clip(x(), y(), w(), h());

   if (value() && down_image_)
      down_image_->draw(x(), y(), w()-3, h()-3, 0, 0);
   else if (focus_highlight_ && focus_image_)
      focus_image_->draw(x(), y(), w()-3, h()-3, 0, 0);
   else if (up_image_)
      up_image_->draw(x(), y(), w()-3, h()-3, 0, 0);

   draw_label();

   //if (Fl::focus() == this && focus_image_)
   //   focus_image_->draw(x(), y(), w()-3, h()-3, 0, 0);

   fl_pop_clip();
}

/**
 * Event handler.
 * @param event Event
 * @return Standard event handler return value
 */
int Fl_SVG_Button::handle(int event)
{
   switch (event)
   {
   case FL_PUSH:
   case FL_DRAG:
   case FL_RELEASE:
      {
         Fl_SVG_Image* img = value() ? down_image_ : up_image_;

         if (!img->hit(Fl::event_x() - x(), Fl::event_y() - y(), 120))
         {
            value(0);
            return 0;
         }
         break;
      }
   case FL_MOVE:
      if (focus_image_)
      {
         if (up_image_->hit(Fl::event_x() - x(), Fl::event_y() - y(), 120))
         {
            if (!focus_highlight_)
            {
               focus_highlight_ = true;
               redraw();
            }
         }
         else if (focus_highlight_)
         {
            focus_highlight_ = false;
            redraw();
         }
      }
      break;
   case FL_LEAVE:
      if (focus_highlight_)
      {
         focus_highlight_ = false;
         redraw();
      }
      break;
   default:
      break;
   }

   return Fl_Button::handle(event);
}

