//
// This file is part of the FLTK Imago project
// Copyright (C) 2002-2005 Chris Osgood
// http://imago.functionalfuture.com
//
// "$Id: Fl_SVG_Image.cxx 80 2005-03-29 17:26:55Z chris $"
//
// Fl_SVG_Image routines.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "imago at functionalfuture.com"
//

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "Fl_SVG_Image.H"
#include <fcntl.h>
#include <sys/stat.h>
#include <zlib.h>
#include <string.h>
#include <svg-cairo.h>

#ifdef _WIN32
#include <io.h>
#else
#include <unistd.h>
#endif

/**
 * Constructor for user provided data.  The main difference between this and
 * a normal image is that SVG images do not have any specific size and the
 * size can be specified here.
 * @param bits SVG file data. This can be SVG XML text or SVGZ gzip format.
 * @param len Length of bits
 * @param W Initial width
 * @param H Initial height
 * @param c Background color (used to blend with translucent areas of image)
 */
Fl_SVG_Image::Fl_SVG_Image(const char* bits, size_t len, int W, int H, Fl_Color c)
   : Fl_RGB_Image(0,W,H,4,0), svgcontext_(0), draw_width_(0), draw_height_(0), bgcolor_(c), draw_bg_(c)
{
   if (svgdata(bits, len) != 0)
   {
      destroy();
      return;
   }
}

/**
 * Constructor for reading from file.
 * @param filename SVG file to read.  Supports SVGZ gzip or SVG text format.
 * @param W Initial width
 * @param H Initial height
 * @param c Background color (used to blend with translucent areas of image)
 */
Fl_SVG_Image::Fl_SVG_Image(const char* filename, int W, int H, Fl_Color c)
   : Fl_RGB_Image(0,W,H,4,0), svgcontext_(0), draw_width_(0), draw_height_(0), bgcolor_(c), draw_bg_(c)
{
   int fd = open(filename, O_RDONLY);
   if (fd == -1)
      return;

   char* bits = 0;

   try
   {
      struct stat stat;
      if (fstat(fd, &stat) != 0)
         throw(0);

      bits = new char[stat.st_size];
      if (!bits)
         throw(0);
   
      if (read(fd, bits, stat.st_size) != stat.st_size)
         throw(0);
   
      if (svgdata(bits, stat.st_size) != 0)
         throw(0);
   }
   catch(int)
   {
      destroy();
   }

   if (bits)
      delete [] bits;

   close(fd);
}

/**
 * Destructor
 */
Fl_SVG_Image::~Fl_SVG_Image()
{
   destroy();
}

/**
 * This "cleans" the current state.  Freeing any memory, etc.
 */
void Fl_SVG_Image::destroy()
{
   if (svgcontext_)
   {
      svg_cairo_destroy((svg_cairo_t*)svgcontext_);
      svgcontext_ = 0;
   }

   if (array && alloc_array)
   {
      delete [] array;
      array = 0;
      alloc_array = 0;
   }

   draw_width_ = 0;
   draw_height_ = 0;
}

/**
 * Parses the SVG data to be used in generating the image.
 * @param data SVG text (gzip or plain text XML)
 * @param len Length of SVG text
 * @return 0 on success
 */
int Fl_SVG_Image::svgdata(const char* data, size_t len)
{
   int rv = 0;
   bool free_mem = false;
   const char* decompressed = 0;

   try
   {
      // Get rid of the current state
      destroy();
   
      if (svg_cairo_create((svg_cairo_t**)&svgcontext_) != 0)
         return -1;
   
      decompressed = decompGZ(data, len);
      if (!decompressed)
         decompressed = data;
      else
      {
         free_mem = true;
         len = strlen(decompressed);
      }
   
      if (svg_cairo_parse_buffer((svg_cairo_t*)svgcontext_, decompressed, len) != 0)
      {
         rv = -1;
         throw(0);
      }
   
      svg_cairo_get_size((svg_cairo_t*)svgcontext_, &pt_width_, &pt_height_);
   
      if (!w())
         w(pt_width_);
   
      if (!h())
         h(pt_height_);
   }
   catch(int)
   {
   }

   if (decompressed && free_mem)
      delete [] decompressed;

   return rv;
}

/**
 * FLTK draw() method.  Rebuilds the image if the dimensions have changed.
 * SVG allows clean scaling to any size.
 *
 * This caches the generated image so that the SVG data does not need to be
 * reparsed and rescaled every time the image is drawn.
 * @param XP X position
 * @param YP Y position
 * @param WP Draw width
 * @param HP Draw height
 * @param cx ?? fltk option.. maybe clip x?
 * @param cy ?? fltk option.. maybe clip y?
 */
void Fl_SVG_Image::draw(int XP, int YP, int WP, int HP, int cx, int cy)
{
   if (WP <=0 || HP <= 0)
      return;

   if (!svgcontext_)
   {
      draw_empty(XP, YP);
      return;
   }

   // Check to see if cached version matches current size
   if (draw_width_ != WP || draw_height_ != HP || draw_bg_ != bgcolor_)
   {
      uncache();

      w(WP); h(HP);

      if (array && alloc_array)
         delete [] array;

      size_t array_size = WP * HP * d();
      array = new uchar[array_size];

      data((const char **)&array, 1); // What does this do?

      if (!array)
      {
         // Oops, no memory
         alloc_array = 0;
         draw_empty(XP, YP);
         return;
      }

      memset(const_cast<uchar*>(array), 0x00, array_size);
      alloc_array = 1;
   
      // Cairo stuff
      int base_width, base_height;
      svg_cairo_get_size((svg_cairo_t*)svgcontext_, &base_width, &base_height);
      cairo_t* render_context = cairo_create();
      cairo_scale(render_context, (double)WP/(double)base_width, (double)HP/(double)base_height);
      cairo_set_target_image(render_context, (char*)array, CAIRO_FORMAT_ARGB32, WP, HP, WP * 4);
      svg_cairo_status_t status = svg_cairo_render((svg_cairo_t*)svgcontext_, render_context);
      cairo_destroy(render_context);

      if (status != 0)
      {
         draw_empty(XP, YP);
         return;
      }

      draw_width_ = WP;
      draw_height_ = HP;
      draw_bg_ = bgcolor_;

      // Currently Cairo renders to a funky BGRA format on little endian
      // machines and ARGB on big endian machines.  Neither of which is what we
      // want.  This swaps the bytes around.  Hopefully this can be removed at
      // some point.
      // libpixman has been hacked for little endian platforms so this code is
      // no longer needed there.  Big endian machines still have to do some byte
      // twiddling.
      // BEGIN TEMPORARY HACK
//#ifdef __APPLE__
      {
         int i, j;
         uchar* pos = const_cast<uchar*>(array), temp;
         for (i = 0; i < HP; i++)
         {
            for (j = 0; j < WP; j++)
            {
#ifdef __APPLE__
               // Convert ARGB to RGBA
               temp = pos[0];
               (*((unsigned int*)pos)) <<= 8;
               pos[3] = temp;
#else
               // Convert BGRA to RGBA
               temp = pos[0];
               pos[0] = pos[2];
               pos[2] = temp;
#endif
               pos += 4;
            }
         }
      }
//#endif // __APPLE__
      // END TEMPORARY HACK

#ifndef HARDWARE_ALPHA
      // This does a software alpha blending with the background for pixels
      // that are not fully transparent or opaque.  fl_read_image appears to
      // return funky results with some X11 drivers and on MacOS.  That code
      // is disabled and we just use bgcolor_ for blending.
      //
      // FLTK is supposed to support translucency on some platforms but none
      // of the platforms I tested did anything more than the ugly masking
      // effect.

// fl_read_image code (doesn't work on all platforms)
//      {
//         int i, j;
//         int clip_x, clip_y, clip_w, clip_h;
//
//         // Get current clip box then read that area of the screen
//         fl_clip_box(XP, YP, WP, HP, clip_x, clip_y, clip_w, clip_h);
//
//         uchar* bg_image = new uchar[clip_w * clip_h * 3];
//         if (!bg_image)
//         {
//            draw_empty(XP, YP);
//            return;
//         }
//
//         fl_read_image(bg_image, clip_x, clip_y, clip_w, clip_h, 0);
//   
//         uchar* pos_bg = bg_image;
//         uchar* pos = const_cast<uchar*>(array);
//#        define ALPHA_INDEX 3
//#        define char_max(x) ((uchar)(((x)>0xFF)?0xFF:(x)))
//   
//         for (i = 0; i < HP; i++)
//         for (j = 0; j < WP; j++)
//         {
//            if (pos[ALPHA_INDEX] != 0xFF && pos[ALPHA_INDEX] != 0x00)
//            {
//               float alpha1 = (float)pos[ALPHA_INDEX] / 255.0f;
//               float alpha2 = 1.0f - alpha1;
//               *pos = char_max((int)((float)*pos * alpha1 + (float)*pos_bg * alpha2)); // red
//               pos++; pos_bg++;
//               *pos = char_max((int)((float)*pos * alpha1 + (float)*pos_bg * alpha2)); // green
//               pos++; pos_bg++;
//               *pos = char_max((int)((float)*pos * alpha1 + (float)*pos_bg * alpha2)); // blue
//               pos++; pos_bg++;
//               *pos = 0xFF; // Alpha value
//               pos++;
//               continue;
//            }
//
//            pos += 4;
//         }
//   
//         delete [] bg_image;
//      }

      // bgcolor_ code
      {
         int i, j;
         uchar r, g, b;

         Fl::get_color(bgcolor_, r, g, b);

         uchar* pos = const_cast<uchar*>(array);
#        define ALPHA_INDEX 3
#        define char_max(x) ((uchar)(((x)>0xFF)?0xFF:(x)))
   
         for (i = 0; i < HP; i++)
         for (j = 0; j < WP; j++)
         {
            if (pos[ALPHA_INDEX] != 0xFF && pos[ALPHA_INDEX] != 0x00)
            {
               float alpha1 = (float)pos[ALPHA_INDEX] / 255.0f;
               float alpha2 = 1.0f - alpha1;
               *pos = char_max((int)((float)*pos * alpha1 + (float)r * alpha2)); // red
               pos++;
               *pos = char_max((int)((float)*pos * alpha1 + (float)g * alpha2)); // green
               pos++;
               *pos = char_max((int)((float)*pos * alpha1 + (float)b * alpha2)); // blue
               pos++;
               *pos = 0xFF; // Alpha value
               pos++;
               continue;
            }

            pos += 4;
         }
      }
#endif // HARDWARE_ALPHA
   }

   Fl_RGB_Image::draw(XP, YP, WP, HP, cx, cy);
}

// Internal callback function for zlib
static void* zalloc_func(void*, unsigned int items, unsigned int size)
{
   return new char[items * size];
}

// Internal callback function for zlib
static void zfree_func(void*, void* address)
{
   delete [] (char*)address;
}

/**
 * GZip decompressor used for SVGZ files
 * @param in Input data (compressed with gzip)
 * @param size Length of input data
 * @return Returns SVG text file (be sure to delete [] it) or null if error
 */
char* Fl_SVG_Image::decompGZ(const char* in, size_t size)
{
   uchar t_buf[32768]; // Fairly large chunk size.. SVG files can be really big
   uchar* out_buf = 0;
   size_t alloc_size = 0;
   z_stream s;
   int rv;

   s.zalloc = zalloc_func;
   s.zfree = zfree_func;
   s.opaque = 0;
   s.next_in = (Bytef*)in;
   s.avail_in = size;

   rv = inflateInit2(&s, 0x2F);
   if (rv != Z_OK)
      return 0;

   while(1)
   {
      s.next_out = t_buf;
      s.avail_out = sizeof(t_buf);
   
      rv = inflate(&s, Z_FULL_FLUSH);

      size_t num = sizeof(t_buf) - s.avail_out;
      uchar* new_buf = new uchar[alloc_size + num + 1];
      if (!new_buf)
         break;

      if (alloc_size)
         memcpy(new_buf, out_buf, alloc_size);

      memcpy(&new_buf[alloc_size], t_buf, num);

      if (out_buf)
         delete [] out_buf;

      out_buf = new_buf;
      alloc_size += num;

      if (rv == Z_STREAM_END)
      {
         inflateEnd(&s);
         out_buf[alloc_size] = 0x00;
         return reinterpret_cast<char*>(out_buf);
      }
      else if (rv != Z_OK)
         break;
   }

   if (out_buf)
      delete [] out_buf;

   inflateEnd(&s);

   return 0;
}

/**
 * Tests to see if a particular x/y coordinate "hit" the image in a non-
 * transparent part.  It might be worthwhile to make this inline by
 * moving it to the header.
 * @param at_x X coordinate
 * @param at_y Y coordinate
 * @param thresh Alpha threshold (0-255; 255 is fully opaque; 1 is default)
 */
bool Fl_SVG_Image::hit(int at_x, int at_y, int thresh)
{
   if (d() < 4)
      return true;
   else if (at_x < 0 || at_x >= w() || at_y < 0 || at_y >= h())
      return false;
   else if (array && array[(at_y * (w()*d()) + (at_x*d()) + (d()-1))] > thresh)
      return true;
   else
      return false;
}

